<?php

namespace Acms\Plugins\ShoppingCart;

use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use Acms\Plugins\ShoppingCart\Services\TaxService;
use App;
use Media;

class Corrector
{
    /**
     * @var TaxService
     */
    protected $TaxService;

    /**
     * @var ConfigRepository
     */
    protected $Config;

    /**
     * constructor
     */
    public function __construct()
    {
        $this->TaxService = App::make('shopping-cart.service.tax');
        $this->Config = App::make('shopping-cart.repository.config');
    }

    /**
     * 管理画面用
     */
    public function getMediaThumbFromMid($mid)
    {
        $media = Media::getMedia($mid);

        if (array_key_exists('path', $media)) {
            return Media::getImageThumbnail($media['path']);
        }

        return "";
    }

    /**
     * 税抜き価格を税込価格にする(数字は税抜きであることが前提)
     *
     * @param  string $price 税抜き価格
     * @param  array  $args 税率(第一引数に税率を渡す)
     *
     * @return string 税込価格
     */
    public function convertToIncludingTax($price, $args = [])
    {
        $taxRate = isset($args[0]) ? doubleval($args[0]) : false;
        if (empty($taxRate)) {
            return $price;
        }

        $includingTax = $this->TaxService->calcIncludingTax(intval($price), $taxRate);
        return "{$includingTax}";
    }

    /**
     * 合計金額（単価 * 数量）を計算する
     *
     * @param  string $price 単価
     * @param  array  $args 数量と税率(第一引数に数量、第二引数に税率(省略可)を指定する)
     *
     * @return string 小計
     */
    public function convertToTotalPrice($price, $args = [])
    {
        $quantity = intval(isset($args[0]) ? $args[0] : 0);

        $sum = $this->TaxService->calcTotalPrice(
            intval($price),
            $quantity,
            !empty($args[1]) ? doubleval($args[1]) : null
        );

        return "{$sum}";
    }

    /**
     * 価格に対する消費税を計算する
     *
     * @param string $price 単価
     * @param array $args 税率(第一引数に指定)
     *
     * @return string 消費税
     */
    public function convertToTax($price, $args = [])
    {
        $taxRate = isset($args[0]) ? doubleval($args[0]) : false;
        if (empty($taxRate)) {
            return $price;
        }

        $tax = $this->TaxService->calcTax(intval($price), $taxRate);
        return "{$tax}";
    }

    /**
     * 配送グループIDを配送グループ名に変換する
     *
     * @param string $id 配送グループID
     *
     * @return string 配送グループ名
     */
    public function shippingGroupName(string $id): string
    {
        return $this->Config->findShippingGroup($id)->getName();
    }
}
