<?php

namespace Acms\Plugins\ShoppingCart\Entities;

/**
 * カート
 */
class Cart
{
    /**
     * 受注中の注文情報ID
     *
     * @var string|null
     */
    private $orderProcessingId;

    /**
     * カート内商品
     *
     * @var Item[]
     */
    private $Items = [];

    public function getOrderProcessingId(): ?string
    {
        return $this->orderProcessingId;
    }

    public function setOrderProcessingId(?string $orderProcessingId): void
    {
        $this->orderProcessingId = $orderProcessingId;
    }

    public function getItems(): array
    {
        return $this->Items;
    }

    public function setItems(array $Items = []): void
    {
        $this->Items = $Items;
    }

    public function addItem(Item $Item): void
    {
        $this->Items[] = $Item;
    }

    public function removeItem(Item $Item): void
    {
        $this->Items = array_values(
            array_filter(
                $this->Items,
                function (Item $i) use ($Item) {
                    return $i->getEntry()->getId() !== $Item->getEntry()->getId();
                }
            )
        );
    }

    /**
     * エントリIDからアイテムを取得する
     *
     * @param int $entryId
     *
     * @return Item|null
     */
    public function getItemByEntryId(int $entryId): ?Item
    {
        $itemAry = array_filter(
            $this->Items,
            function (Item $Item) use ($entryId) {
                return $Item->getEntry()->getId() === $entryId;
            }
        );

        return empty($itemAry) ? null : array_values($itemAry)[0];
    }

    public function __clone()
    {
        $this->Items = array_map(
            function (Item $Item) {
                return clone $Item;
            },
            $this->Items
        );
    }
}
