<?php

namespace Acms\Plugins\ShoppingCart\Entities;

use Acms\Plugins\ShoppingCart\Entities\Master\EntryApproval;
use Acms\Plugins\ShoppingCart\Entities\Master\EntryStatus;

/**
 * エントリー
 */
class Entry
{
    /**
     * ID
     */
    private int $id;

    /**
     * コード
     */
    private string $code;

    /**
     * ステータス
     */
    private string $status = EntryStatus::OPEN;

    /**
     * 承認ステータス
     */
    private string $approval = EntryApproval::NONE;

    /**
     * フォームステータス
     */
    private string $formStatus;

    /**
     * ソート番号
     */
    private int $sort;

    /**
     * ユーザーのソート番号
     */
    private int $userSort;

    /**
     * カテゴリーのソート番号
     */
    private int $categorySort;

    /**
     * タイトル
     */
    private string $title;

    /**
     * リンク
     */
    private string $link;

    /**
     * 日付
     */
    private \DateTimeImmutable $datetime;

    /**
     * 公開日時
     */
    private \DateTimeImmutable $startDatetime;

    /**
     * 掲載終了日時
     */
    private \DateTimeImmutable $endDatetime;

    /**
     * 作成日時
     */
    private \DateTimeImmutable $postedDatetime;

    /**
     * 更新日時
     */
    private \DateTimeImmutable $updatedDatetime;

    /**
     * 概要
     */
    private ?int $summaryRange = null;

    /**
     * インデキシング
     */
    private string $indexing = 'on';

    /**
     *  メインイメージ
     */
    private ?string $primaryImage = null;

    /**
     * リビジョンID
     */
    private int $currentRevId;

    /**
     * 最後に更新したユーザーID
     */
    private string $lastUpdateUserId;

    /**
     *  ハッシュ
     */
    private string $hash;

    /**
     *  カテゴリーID
     */
    private ?int $categoryId = null;

    /**
     *  ユーザーID
     */
    private int $userId;

    /**
     *  フォームID
     */
    private int $formId;

    /**
     *  ブログID
     */
    private int $blogId;

    /**
     *  削除したユーザーのID
     */
    private ?int $deleteUid = null;

    /**
     * エントリーフィールド
     */
    private \Field $Field;

    /**
     * 配送グループ
     *
     * @var ShippingGroup|null
     */
    private $ShippingGroup;


    public function getId(): int
    {
        return $this->id;
    }

    public function setId(int $id): void
    {
        $this->id = $id;
    }

    public function getCode(): string
    {
        return $this->code;
    }

    public function setCode(string $code): void
    {
        $this->code = $code;
    }

    public function getStatus(): string
    {
        return $this->status;
    }

    public function setStatus(string $status): void
    {
        $this->status = $status;
    }

    public function getApproval(): string
    {
        return $this->approval;
    }

    public function setApproval(string $approval): void
    {
        $this->approval = $approval;
    }

    public function getFormStatus(): string
    {
        return $this->formStatus;
    }

    public function setFormStatus(string $formStatus): void
    {
        $this->formStatus = $formStatus;
    }

    public function getSort(): string
    {
        return $this->sort;
    }

    public function setSort(string $sort): void
    {
        $this->sort = $sort;
    }

    public function getUserSort(): int
    {
        return $this->userSort;
    }

    public function setUserSort(int $userSort): void
    {
        $this->userSort = $userSort;
    }

    public function getCategorySort(): int
    {
        return $this->categorySort;
    }

    public function setCategorySort(int $categorySort): void
    {
        $this->categorySort = $categorySort;
    }

    public function getTitle(): string
    {
        return $this->title;
    }

    public function setTitle(string $title): void
    {
        $this->title = $title;
    }

    public function getLink(): string
    {
        return $this->link;
    }

    public function setLink(string $link): void
    {
        $this->link = $link;
    }

    public function getDatetime(): \DateTimeImmutable
    {
        return $this->datetime;
    }

    public function setDatetime(\DateTimeImmutable $datetime): void
    {
        $this->datetime = $datetime;
    }

    public function getStartDatetime(): \DateTimeImmutable
    {
        return $this->startDatetime;
    }

    public function setStartDatetime(\DateTimeImmutable $startDatetime): void
    {
        $this->startDatetime = $startDatetime;
    }

    public function getEndDatetime(): \DateTimeImmutable
    {
        return $this->endDatetime;
    }

    public function setEndDatetime(\DateTimeImmutable $endDatetime): void
    {
        $this->endDatetime = $endDatetime;
    }

    public function getPostedDatetime(): \DateTimeImmutable
    {
        return $this->postedDatetime;
    }

    public function setPostedDatetime(\DateTimeImmutable $postedDatetime): void
    {
        $this->postedDatetime = $postedDatetime;
    }

    public function getUpdatedDatetime(): \DateTimeImmutable
    {
        return $this->updatedDatetime;
    }

    public function setUpdatedDatetime(\DateTimeImmutable $updatedDatetime): void
    {
        $this->updatedDatetime = $updatedDatetime;
    }

    public function getSummaryRange(): ?int
    {
        return $this->summaryRange;
    }

    public function setSummaryRange(?int $summaryRange = null): void
    {
        $this->summaryRange = $summaryRange;
    }

    public function getIndexing(): string
    {
        return $this->indexing;
    }

    public function setIndexing(string $indexing): void
    {
        $this->indexing = $indexing;
    }

    public function getPrimaryImage(): ?int
    {
        return $this->primaryImage;
    }

    public function setPrimaryImage(?int $primaryImage = null): void
    {
        $this->primaryImage = $primaryImage;
    }

    public function getCurrentRevId(): int
    {
        return $this->currentRevId;
    }

    public function setCurrentRevId(int $currentRevId): void
    {
        $this->currentRevId = $currentRevId;
    }

    public function getLastUpdateUserId(): int
    {
        return $this->lastUpdateUserId;
    }

    public function setLastUpdateUserId(int $lastUpdateUserId): void
    {
        $this->lastUpdateUserId = $lastUpdateUserId;
    }

    public function getHash(): string
    {
        return $this->hash;
    }

    public function setHash(string $hash): void
    {
        $this->hash = $hash;
    }

    public function getCategoryId(): ?int
    {
        return $this->categoryId;
    }

    public function setCategoryId(?int $categoryId = null): void
    {
        $this->categoryId = $categoryId;
    }

    public function getUserId(): int
    {
        return $this->userId;
    }

    public function setUserId(int $userId): void
    {
        $this->userId = $userId;
    }

    public function getFormId(): int
    {
        return $this->formId;
    }

    public function setFormId(int $formId): void
    {
        $this->formId = $formId;
    }

    public function getBlogId(): int
    {
        return $this->blogId;
    }

    public function setBlogId(int $blogId): void
    {
        $this->blogId = $blogId;
    }

    public function getDeleteUid(): ?int
    {
        return $this->deleteUid;
    }

    public function setDeleteUid(?int $deleteUid = null): void
    {
        $this->deleteUid = $deleteUid;
    }

    public function getField(): \Field
    {
        return $this->Field;
    }

    public function setField(\Field $Field): void
    {
        $this->Field = $Field;
    }

    public function getShippingGroup(): ?ShippingGroup
    {
        return $this->ShippingGroup;
    }

    public function setShippingGroup(?ShippingGroup $ShippingGroup): void
    {
        $this->ShippingGroup = $ShippingGroup;
    }
}
