<?php

namespace Acms\Plugins\ShoppingCart\Entities;

use Field;

/**
 * 商品情報
 */
class Item
{
    /**
     * エントリー
     *
     * @var Entry
     */
    private $Entry;

    /**
     * 数量
     *
     * @var int
     */
    private $quantity = 0;

    /**
     * カートフィールド
     *
     * @var Field
     */
    private $Field;

    public function getEntry(): Entry
    {
        return $this->Entry;
    }

    public function setEntry(Entry $Entry): void
    {
        $this->Entry = $Entry;
    }

    public function getQuantity(): int
    {
        return $this->quantity;
    }

    public function setQuantity(?int $quantity): void
    {
        $this->quantity = $quantity;
    }

    public function getField(): Field
    {
        if (is_null($this->Field)) {
            return new Field();
        }
        return $this->Field;
    }

    public function setField(Field $Field): void
    {
        $this->Field = $Field;
    }

    /**
     * 在庫管理が有効かどうか
     *
     * @return bool
     */
    public function isStockManagementEnabled(): bool
    {
        return is_numeric($this->getEntry()->getField()->get('item_stock'));
    }

    /**
     * 数量が在庫数を超えているかどうか
     *
     * @return bool
     */
    public function isStockShortage(): bool
    {
        if (!$this->isStockManagementEnabled()) {
            return false;
        }

        return intval($this->getEntry()->getField()->get('item_stock')) < $this->getQuantity();
    }

    /**
     * 数量を追加する
     *
     * @param int $quantity
     */
    public function addQuantity(int $quantity): void
    {
        $this->quantity += $quantity;
    }
}
