<?php

namespace Acms\Plugins\ShoppingCart\Entities;

use Acms\Plugins\ShoppingCart\Entities\TaxAmount;
use Acms\Plugins\ShoppingCart\Entities\ShippingGroup;

/**
 * オーダー
 */
class Order
{
    /**
     * 決済プロセス中のオーダーID（決済フォームと注文を紐付けるために使用する）
     *
     * @var string
     */
    private $processingId;

    /**
     * 配送地域
     *
     * @var string|null
     */
    private $deliveryArea;

    /**
     * 小計
     *
     * @var int
     */
    private $subtotal = 0;


    /**
     * 送料合計
     *
     * @var int
     */
    private $shippingFee = 0;

    /**
     * 決済手数料
     *
     * @var int
     */
    private $commissionPaid = 0;

    /**
     * 追加手数料
     *
     * @var int
     */
    private $additionalFee = 0;

    /**
     * 合計金額
     *
     * @var int
     */
    private $total = 0;

    /**
     * 注文商品
     *
     * @var OrderItem[]
     */
    private $OrderItems = [];

    /**
     * 配送グループ
     *
     * @var ShippingGroup|null
     */
    private $ShippingGroup;

    /**
     * 消費税
     *
     * @var TaxAmount[]
     */
    private $TaxAmounts = [];

    /**
     * 決済方法
     *
     * @var Payment|null
     */
    private $Payment;

    /**
     * 追加手数料
     *
     * @var Addition[]
     */
    private $Additions = [];


    public function getProcessingId(): string
    {
        return $this->processingId;
    }

    public function setProcessingId(string $processingId): void
    {
        $this->processingId = $processingId;
    }

    public function getDeliveryArea(): ?string
    {
        return $this->deliveryArea;
    }

    public function setDeliveryArea(?string $deliveryArea): void
    {
        $this->deliveryArea = $deliveryArea;
    }

    public function getSubtotal(): int
    {
        return $this->subtotal;
    }

    public function setSubtotal(int $subtotal = 0): void
    {
        $this->subtotal = $subtotal;
    }


    public function getShippingFee(): int
    {
        return $this->shippingFee;
    }

    public function setShippingFee(int $shippingFee): void
    {
        $this->shippingFee = $shippingFee;
    }

    public function getCommissionPaid(): int
    {
        return $this->commissionPaid;
    }

    public function setCommissionPaid(int $commissionPaid): void
    {
        $this->commissionPaid = $commissionPaid;
    }

    public function getAdditionalFee(): int
    {
        return $this->additionalFee;
    }

    public function setAdditionalFee(int $additionalFee): void
    {
        $this->additionalFee = $additionalFee;
    }

    public function getTotal(): int
    {
        return $this->total;
    }

    public function setTotal(int $total): void
    {
        $this->total = $total;
    }

    public function getOrderItems(): array
    {
        return $this->OrderItems;
    }

    public function setOrderItems(array $OrderItems = []): void
    {
        $this->OrderItems = $OrderItems;
    }

    public function getShippingGroup(): ShippingGroup
    {
        return $this->ShippingGroup;
    }

    public function setShippingGroup(?ShippingGroup $ShippingGroup): void
    {
        $this->ShippingGroup = $ShippingGroup;
    }

    public function getTaxAmounts(): array
    {
        return $this->TaxAmounts;
    }

    public function setTaxAmounts(array $TaxAmounts = []): void
    {
        $this->TaxAmounts = $TaxAmounts;
    }

    public function getPayment(): ?Payment
    {
        return $this->Payment;
    }

    public function setPayment(?Payment $Payment): void
    {
        $this->Payment = $Payment;
    }

    public function getAdditions(): array
    {
        return $this->Additions;
    }

    public function setAdditions(array $Additions = []): void
    {
        $this->Additions = $Additions;
    }

    /**
     * 在庫管理が有効な注文商品を取得する
     *
     * @return OrderItem[]
     */
    public function getStockManagementItems(): array
    {
        return array_values(
            array_filter(
                $this->OrderItems,
                function (OrderItem $OrderItem) {
                    return $OrderItem->isStockManagementEnabled();
                }
            )
        );
    }
}
