<?php

namespace Acms\Plugins\ShoppingCart\Entities;

/**
 * 注文商品情報
 */
class OrderItem
{
    /**
     * 数量
     *
     * @var int
     */
    private int $quantity = 0;

    /**
     * 単価
     *
     * @var int
     */
    private int $price = 0;


    /**
     * 消費税率
     *
     * @var double|null
     */
    private float $taxRate;

    /**
     * 消費税
     *
     * @var int
     */
    private int $tax;

    /**
     * 配送グループ
     *
     * @var ShippingGroup|null
     */
    private $ShippingGroup;

    /**
     * エントリー
     *
     * @var Entry
     */
    private Entry $Entry;

    /**
     * オーダーアイテムフィールド
     *
     * @var \Field
     */
    private $Field;

    public function getQuantity(): int
    {
        return $this->quantity;
    }

    public function setQuantity(?int $quantity): void
    {
        $this->quantity = $quantity;
    }

    public function getPrice(): ?int
    {
        return $this->price;
    }

    public function setPrice(?int $price): void
    {
        $this->price = $price;
    }

    public function getTaxRate(): ?float
    {
        return $this->taxRate;
    }

    public function setTaxRate(?float $taxRate): void
    {
        $this->taxRate = $taxRate;
    }

    public function getTax(): ?int
    {
        return $this->tax;
    }

    public function setTax(int $tax): void
    {
        $this->tax = $tax;
    }

    public function getShippingGroup(): ?ShippingGroup
    {
        return $this->ShippingGroup;
    }

    public function setShippingGroup(?ShippingGroup $ShippingGroup): void
    {
        $this->ShippingGroup = $ShippingGroup;
    }

    public function getEntry(): Entry
    {
        return $this->Entry;
    }

    public function setEntry(Entry $Entry): void
    {
        $this->Entry = $Entry;
    }

    public function getField(): \Field
    {
        if (is_null($this->Field)) {
            return new \Field();
        }
        return $this->Field;
    }

    public function setField(\Field $Field): void
    {
        $this->Field = $Field;
    }

    /**
     * 在庫管理が有効かどうか
     *
     * @return bool
     */
    public function isStockManagementEnabled(): bool
    {
        return is_numeric($this->getEntry()->getField()->get('item_stock'));
    }

    /**
     * 数量が在庫数を超えているかどうか
     *
     * @return bool
     */
    public function isStockShortage(): bool
    {
        if (!$this->isStockManagementEnabled()) {
            return false;
        }

        return intval($this->getEntry()->getField()->get('item_stock')) < $this->getQuantity();
    }
}
