<?php

namespace Acms\Plugins\ShoppingCart\GET\Addition;

use Acms\Plugins\ShoppingCart\Entities\Master\Status;
use Acms\Plugins\ShoppingCart\Entities\Addition;
use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use App;
use ACMS_GET;
use ACMS_Corrector;
use Template;

/**
 * 決済フォームで選択可能な追加手数料を表示するモジュール
 */
class Select extends ACMS_GET
{
    /**
     * @var ConfigRepository
     */
    protected ConfigRepository $Config;

    public function get()
    {
        $Tpl = new Template($this->tpl, new ACMS_Corrector());
        $this->buildModuleField($Tpl);

        $this->Config = App::make('shopping-cart.repository.config');

        $Additions = $this->getAdditions();

        if (empty($Additions)) {
            return $Tpl->render([
                'notFound' => new \stdClass()
            ]);
        }

        return $Tpl->render([
            'addition' => $this->buildAdditions($Additions)
        ]);
    }


    /**
     * 決済フォームで利用可能な追加手数料を取得する
     *
     * @return Addition[]
     */
    protected function getAdditions(): array
    {
        return $this->Config->findAdditions(null, Status::OPEN);
    }

    /**
     * 追加手数料のテンプレートを組み立て
     *
     * @param Addition[] $Additions
     *
     * @return array
     */
    protected function buildAdditions(array $Additions): array
    {
        return array_map(
            function (Addition $Addition) {
                return [
                    'id' => $Addition->getId(),
                    'status' => $Addition->getStatus(),
                    'name' => $Addition->getName(),
                    'fee' => $Addition->getFee()
                ];
            },
            $Additions
        );
    }
}
