<?php

namespace Acms\Plugins\ShoppingCart\GET\Order;

use Acms\Plugins\ShoppingCart\Entities\Addition;
use Acms\Plugins\ShoppingCart\Entities\Item;
use Acms\Plugins\ShoppingCart\Entities\Entry;
use Acms\Plugins\ShoppingCart\Entities\Order;
use Acms\Plugins\ShoppingCart\Entities\OrderItem;
use Acms\Plugins\ShoppingCart\Entities\Payment;
use Acms\Plugins\ShoppingCart\Entities\ShippingGroup;
use Acms\Plugins\ShoppingCart\Entities\TaxAmount;
use Acms\Plugins\ShoppingCart\Repositories\CartRepository;
use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use Acms\Plugins\ShoppingCart\Repositories\EntryRepository;
use Acms\Plugins\ShoppingCart\Services\ShoppingService;
use Acms\Plugins\ShoppingCart\Services\TaxService;
use App;
use ACMS_GET;
use ACMS_Corrector;
use Template;
use Field;
use Acms\Services\Facades\Media;
use Acms\Services\Facades\Template as Tpl;

/**
 * 注文内容の金額を表示するモジュール
 */
class Summary extends ACMS_GET
{
    protected const MEDIA_FIELD_PREFIX = 'image';

    /**
     * @var CartRepository
     */
    protected CartRepository $CartRepository;

    /**
     * @var ShoppingService
     */
    protected ShoppingService $ShoppingService;

    /**
     * @var TaxService
     */
    protected TaxService $TaxService;

    /**
     * @var EntryRepository
     */
    protected EntryRepository $EntryRepository;

    /**
     * @var ConfigRepository
     */
    protected ConfigRepository $Config;

    /**
     * ACMS_GETモジュールの$Fieldと変数名が重複するため$Field2としている
     *
     * @var Field
     */
    public Field $Field2;

    /**
     * @var Order
     */
    protected Order $Order;

    /**
     * @var array
     */
    protected array $config;

    /**
     * コンフィグの取得
     *
     * @return array
     */
    protected function initVars()
    {
        return [
            'imageX' => intval(config('order_summary_image_x')),
            'imageY' => intval(config('order_summary_image_y')),
            'imageTrim' => config('order_summary_image_trim'),
            'imageZoom' => config('order_summary_image_zoom'),
            'imageCenter' => config('order_summary_image_center'),
            'mainImageOn' => config('order_summary_image_on'),
        ];
    }

    /**
     * コンフィグのセット
     *
     * @return bool
     */
    protected function setConfig()
    {
        $this->config = $this->initVars();
        return true;
    }

    public function get()
    {
        if (!$this->setConfig()) {
            return '';
        }

        $Tpl = new Template($this->tpl, new ACMS_Corrector());
        $this->buildModuleField($Tpl);

        $this->CartRepository = App::make('shopping-cart.repository.cart');
        $this->ShoppingService = App::make('shopping-cart.service.shopping');
        $this->TaxService = App::make('shopping-cart.service.tax');
        $this->Config = App::make('shopping-cart.repository.config');
        $this->EntryRepository = App::make('shopping-cart.repository.entry');

        /**
         * Set Field
         */
        $this->Field2 = $this->Post->getChild('field');
        $this->Field2->overload($this->Field); // URLコンテキストのFieldを優先

        /**
         * Set Order
         */
        $this->setOrder();

        return $Tpl->render($this->buildTemplate($Tpl));
    }

    /**
     * テンプレートの組み立て
     * @param Template $Tpl
     *
     * @return array
     */
    protected function buildTemplate(Template $Tpl): array
    {
        return [
            'itemsAmount' => count($this->Order->getOrderItems()),
            'quantitiesAmount' => array_reduce(
                $this->Order->getOrderItems(),
                function (int $quantity, OrderItem $OrderItem) {
                    return $quantity + $OrderItem->getQuantity();
                },
                0
            ),
            'deliveryArea' => $this->Order->getDeliveryArea(),
            'subtotal' => $this->Order->getSubtotal(),
            'shippingFee' => $this->Order->getShippingFee(),
            'commissionPaid' => $this->Order->getCommissionPaid(),
            'additionalFee' => $this->Order->getAdditionalFee(),
            'total' => $this->Order->getTotal(),
            'orderItem' => $this->buildOrderItem($Tpl, $this->Order->getOrderItems()),
            'taxAmount' => $this->buildTaxAmount($this->Order->getTaxAmounts()),
            'payment' => (
                !is_null($this->Order->getPayment())
                    ? $this->buildPayment($this->Order->getPayment())
                    : []
            ),
            'addition' => $this->buildAddition($this->Order->getAdditions())
        ];
    }

    /**
     * 注文商品情報ブロックの組み立て
     *
     * @param Template $Tpl
     * @param OrderItem[] $OrderItems
     *
     * @return array
     */
    protected function buildOrderItem(Template $Tpl, array $OrderItems): array
    {
        $eagerLoadingData = $this->eagerLoad();
        return array_map(
            function (OrderItem $OrderItem) use ($Tpl, $eagerLoadingData) {
                return array_merge(
                    [
                        'quantity' => $OrderItem->getQuantity(),
                        'price' => $OrderItem->getPrice(),
                        'taxRate' => $OrderItem->getTaxRate(),
                        'taxPrice' => $OrderItem->getTax(),
                        'totalTaxIncluded' => $this->TaxService->calcTotalPrice(
                            $OrderItem->getPrice(),
                            $OrderItem->getQuantity(),
                            $this->Config->get('shopping-cart_tax-format') === 'extax' ? $OrderItem->getTaxRate() : null
                        ),
                        'shippingGroup' => (
                            !is_null($OrderItem->getShippingGroup())
                                ? $this->buildShippingGroup($OrderItem->getShippingGroup())
                                : []
                        ),
                        'orderItemField' => $this->buildField(
                            $OrderItem->getField(),
                            $Tpl,
                            ['orderItemField', 'orderItem:loop']
                        )
                    ],
                    $this->buildEntry($Tpl, $OrderItem->getEntry(), $eagerLoadingData)
                );
            },
            $OrderItems
        );
    }


    /**
     * 配送グループブロックの組み立て
     *
     * @param ShippingGroup $ShippingGroup
     *
     * @return array
     */
    protected function buildShippingGroup(ShippingGroup $ShippingGroup): array
    {
        return [
            'id' => $ShippingGroup->getId(),
            'status' => $ShippingGroup->getStatus(),
            'name' => $ShippingGroup->getName(),
            'groupFee' => $ShippingGroup->getGroupFee(),
            'calcMethod' => $ShippingGroup->getCalcMethod(),
        ];
    }

    /**
     * 注文商品のエントリーデータの組み立て
     *
     * @param Template $Tpl
     * @param Entry $Entry
     * @param array $eagerLoadingData
     *
     * @return array
     */
    protected function buildEntry(Template $Tpl, Entry $Entry, array $eagerLoadingData): array
    {
        return array_merge(
            [
                'eid' => $Entry->getId(),
                'ecd' => $Entry->getCode(),
                'uid' => $Entry->getUserId(),
                'cid' => $Entry->getCategoryId(),
                'bid' => $Entry->getBlogId(),
                'sort' => $Entry->getSort(),
                'csort' => $Entry->getCategorySort(),
                'usort' => $Entry->getUserSort(),
                'status' => $Entry->getStatus(),
                'title' => addPrefixEntryTitle(
                    $Entry->getTitle(),
                    $Entry->getStatus(),
                    $Entry->getStartDatetime()->format('Y-m-d H:i:s'),
                    $Entry->getEndDatetime()->format('Y-m-d H:i:s'),
                    $Entry->getApproval()
                ),
                'url' => (
                    !empty($Entry->getLink())
                        ? $Entry->getLink()
                        : acmsLink([
                            'bid' => $Entry->getBlogId(),
                            'cid' => $Entry->getCategoryId(),
                            'eid' => $Entry->getId()
                        ])
                ),
                'parmalink' => acmsLink([
                    'bid' => $Entry->getBlogId(),
                    'cid' => $Entry->getCategoryId(),
                    'eid' => $Entry->getId()
                ], false)
            ],
            $this->buildField($Entry->getField(), $Tpl, ['orderItem:loop']),
            $this->buildDate(
                $Entry->getDatetime()->format('Y-m-d H:i:s'),
                $Tpl,
                ['orderItem:loop']
            ),
            $this->buildDate(
                $Entry->getUpdatedDatetime()->format('Y-m-d H:i:s'),
                $Tpl,
                ['orderItem:loop'],
                'udate#'
            ),
            $this->buildDate(
                $Entry->getPostedDatetime()->format('Y-m-d H:i:s'),
                $Tpl,
                ['orderItem:loop'],
                'pdate#'
            ),
            $this->buildDate(
                $Entry->getStartDatetime()->format('Y-m-d H:i:s'),
                $Tpl,
                ['orderItem:loop'],
                'sdate#'
            ),
            $this->buildDate(
                $Entry->getEndDatetime()->format('Y-m-d H:i:s'),
                $Tpl,
                ['orderItem:loop'],
                'edate#'
            ),
            $this->buildImage(
                $Tpl,
                $Entry->getPrimaryImage(),
                $this->config,
                $eagerLoadingData['mainImage']
            ),
        );
    }

    /**
     * 税金ブロックの組み立て
     *
     * @param TaxAmount[] $TaxAmounts
     *
     * @return array
     */
    protected function buildTaxAmount(array $TaxAmounts): array
    {
        return array_map(
            function (TaxAmount $TaxAmount) {
                return [
                    'taxRate' => $TaxAmount->getTaxRate() * 100,  // %に変換
                    'excludingTax' => $TaxAmount->getExcludingTax(),
                    'tax' => $TaxAmount->getTax()
                ];
            },
            $TaxAmounts
        );
    }


    /**
     * 決済方法ブロックの組み立て
     *
     * @param Payment $Payment
     *
     * @return array
     */
    protected function buildPayment(Payment $Payment): array
    {
        return array_merge(
            [
                'block' => $Payment->getBlock(),
                'name' => $Payment->getName(),
                'commissionPaid' => $Payment->getCommissionPaid(),
                'isRuleEnabled' => $Payment->getIsRuleEnabled(),
                'description' => $Payment->getDescription(),
                'mailMessage' => $Payment->getMailMessage(),
            ],
            (!is_null($Payment->getMedia()) ? [
                $this::MEDIA_FIELD_PREFIX => $Payment->getMedia()->getId(),
                $this::MEDIA_FIELD_PREFIX . '@name' => $Payment->getMedia()->getFileName(),
                $this::MEDIA_FIELD_PREFIX . '@extension' => $Payment->getMedia()->getExtension(),
                $this::MEDIA_FIELD_PREFIX . '@fileSize' => $Payment->getMedia()->getFileSize(),
                $this::MEDIA_FIELD_PREFIX . '@caption' => $Payment->getMedia()->getCaption(),
                $this::MEDIA_FIELD_PREFIX . '@link' => $Payment->getMedia()->getLink(),
                $this::MEDIA_FIELD_PREFIX . '@alt' => $Payment->getMedia()->getAlt(),
                $this::MEDIA_FIELD_PREFIX . '@text' => $Payment->getMedia()->getText(),
                $this::MEDIA_FIELD_PREFIX . '@path' => $Payment->getMedia()->getPath(),
                $this::MEDIA_FIELD_PREFIX . '@thumbnail' => Media::getImageThumbnail(
                    $Payment->getMedia()->getPath()
                ),
                $this::MEDIA_FIELD_PREFIX . '@imageSize' => $Payment->getMedia()->getImageSize(),
                $this::MEDIA_FIELD_PREFIX . '@type' => $Payment->getMedia()->getType(),
                $this::MEDIA_FIELD_PREFIX . '@page' => $Payment->getMedia()->getPdfPage(),
                $this::MEDIA_FIELD_PREFIX . '@focalX' => $Payment->getMedia()->getFocalX(),
                $this::MEDIA_FIELD_PREFIX . '@focalY' => $Payment->getMedia()->getFocalY(),
                $this::MEDIA_FIELD_PREFIX . '@width' => $Payment->getMedia()->getWidth(),
                $this::MEDIA_FIELD_PREFIX . '@height' => $Payment->getMedia()->getHeight(),
                $this::MEDIA_FIELD_PREFIX . '@ratio' => $Payment->getMedia()->getRatio(),
            ] : [])
        );
    }

    /**
     * 追加手数料ブロックの組み立て
     *
     * @param Addition[] $Additions
     *
     * @return array
     */
    protected function buildAddition(array $Additions): array
    {
        return array_map(
            function (Addition $Addition) {
                return [
                    'id' => $Addition->getId(),
                    'name' => $Addition->getName(),
                    'fee' => $Addition->getFee()
                ];
            },
            $Additions
        );
    }

    /**
     * 注文商品の取得
     *
     * @return Item[]
     */
    protected function getItems(): array
    {
        if ($this->eid) {
            return $this->getItemsFromEntry($this->eid);
        }

        return $this->getItemsFromCart($this->bid);
    }

    /**
     * カートから注文商品の取得
     * @param int $bid
     *
     * @return Item[]
     */
    protected function getItemsFromCart(int $bid): array
    {
        $Cart = $this->CartRepository->find($bid);
        return $Cart->getItems();
    }

    /**
     * エントリーから注文商品の取得
     * @param int $eid
     *
     * @return Item[]
     */
    protected function getItemsFromEntry(int $eid): array
    {
        $Entry = $this->EntryRepository->findById($eid);
        $quantity = intval($this->Field2->get('quantity', 0));

        $Item = new Item();
        $Item->setQuantity($quantity);
        $Item->setEntry($Entry);

        return [$Item];
    }

    /**
     * 購入処理中の注文情報を取得する
     *
     * @return Order|null
     */
    protected function loadProcessingOrder(): ?Order
    {
        $processingOrderId = $this->Post->get('processingOrderId', $this->Get->get('processingOrderId'));

        if ($processingOrderId !== '') {
            return $this->ShoppingService->loadProcessingOrder($processingOrderId);
        }

        return null;
    }

    /**
     * 注文情報のセット
     *
     * @return void
     */
    protected function setOrder(): void
    {
        $Order = $this->loadProcessingOrder();
        if (is_null($Order)) {
            // Order_Summaryの役割的に、購入プロセス中の注文データを表示することのみを責務としたいが、
            // 互換性（Ver.2.0.0）の問題で、購入プロセス中の注文データが存在しない場合は注文データを新規作成している。
            $Order = $this->ShoppingService->createOrder($this->getItems(), $this->Field2);
        }

        $this->Order = $Order;
    }

    /**
     * EagerLoading
     *
     * @return array
     */
    protected function eagerLoad(): array
    {
        return [
            'mainImage' => $this->mainImageEagerLoad()
        ];
    }

    /**
     * メイン画像のEagerLoading
     *
     * @return array
     */
    protected function mainImageEagerLoad(): array
    {
        if (!isset($this->config['mainImageOn']) || $this->config['mainImageOn'] === 'on') {
            $mainImages = array_map(
                function (OrderItem $OrderItem) {
                    return [
                        'entry_primary_image' => $OrderItem->getEntry()->getPrimaryImage()
                    ];
                },
                $this->Order->getOrderItems()
            );
            return Tpl::eagerLoadMainImage($mainImages);
        }
        return [];
    }
}
