<?php

namespace Acms\Plugins\ShoppingCart\GET\ShippingGroup;

use Acms\Plugins\ShoppingCart\Entities\Master\Status;
use Acms\Plugins\ShoppingCart\Entities\ShippingGroup;
use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use App;
use ACMS_GET;
use ACMS_Corrector;
use Template;

/**
 * エントリー編集画面で選択可能な配送グループを表示するモジュール
 */
class AdminSelect extends ACMS_GET
{
    /**
     * @var ConfigRepository
     */
    protected ConfigRepository $Config;

    public function get()
    {
        $Tpl = new Template($this->tpl, new ACMS_Corrector());
        $this->buildModuleField($Tpl);

        $this->Config = App::make('shopping-cart.repository.config');

        $ShippingGroups = $this->getShippingGroup();

        if (empty($ShippingGroups)) {
            return $Tpl->render([
                'notFound' => new \stdClass()
            ]);
        }

        return $Tpl->render([
            'shippingGroup' => $this->buildShippingGroup($ShippingGroups)
        ]);
    }


    /**
     * エントリー編集画面で選択可能な配送グループを取得する
     *
     * @return ShippingGroup[]
     */
    protected function getShippingGroup(): array
    {
        return $this->Config->findShippingGroups(null, Status::OPEN);
    }

    /**
     * 配送グループのテンプレートを組み立て
     *
     * @param ShippingGroup[] $ShippingGroups
     *
     * @return array
     */
    protected function buildShippingGroup(array $ShippingGroups): array
    {
        return array_map(
            function (ShippingGroup $ShippingGroup) {
                return [
                    'id' => $ShippingGroup->getId(),
                    'status' => $ShippingGroup->getStatus(),
                    'name' => $ShippingGroup->getName(),
                    'groupFee' => $ShippingGroup->getGroupFee(),
                    'calcMethod' => $ShippingGroup->getCalcMethod(),
                ];
            },
            $ShippingGroups
        );
    }
}
