<?php

namespace Acms\Plugins\ShoppingCart\GET;

use Acms\Plugins\ShoppingCart\Entities\Item;
use Acms\Plugins\ShoppingCart\Entities\ShippingGroup;
use ACMS_GET;
use ACMS_Corrector;
use Template;
use Session;

class ShoppingCart extends ACMS_GET
{
    /**
     * @var \Acms\Services\Session\Engine
     */
    protected $Session;

    /**
     * @var string|null
     */
    protected $block;

    public function get()
    {
        if (!defined('NO_CACHE_PAGE')) {
            define('NO_CACHE_PAGE', 1);
        }
        $this->Session = Session::handle();

        $Tpl = new Template($this->tpl, new ACMS_Corrector());

        $this->block = $this->getBlock();
        $Item = $this->getCartItem();

        if (is_null($Item) && is_null($this->block)) {
            return $Tpl->render([]);
        }

        if (!$this->Post->isValidAll()) {
            return $Tpl->render([
                $this->block => array_merge(
                    ['status' => 'failure'],
                    $this->buildField($this->Post, $Tpl)
                )
            ]);
        }

        return $Tpl->render([
            $this->block => array_merge(
                ['status' => 'success'],
                ['item' => $this->buildItem($Tpl, $Item)]
            )
        ]);
    }

    /**
     * カートアイテムの取得
     *
     * @return Item|null
     */
    protected function getCartItem(): ?Item
    {
        $cartItem = $this->Session->get('cartItem');

        if (empty($cartItem)) {
            return null;
        }

        $this->Session->delete('cartItem');
        $this->Session->save();
        return $cartItem;
    }

    /**
     * ブロックの取得
     *
     * @return string|null
     */
    protected function getBlock(): ?string
    {
        $block = $this->Session->get('block');

        if (empty($block)) {
            return null;
        }

        $this->Session->delete('block');
        $this->Session->save();
        return $block;
    }

    /**
     * カートアイテムブロックの組み立て
     *
     * @param Template $Tpl
     * @param Item $Item
     *
     * @return array
     */
    protected function buildItem(Template $Tpl, Item $Item): array
    {
        return [
            'eid' => $Item->getEntry()->getId(),
            'quantity' => $Item->getQuantity(),
            'cartField' => $this->buildField($Item->getField(), $Tpl, ['cartField', 'item:loop', $this->block]),
            'price' => $Item->getEntry()->getField()->get('item_price'),
            'stock' => $Item->getEntry()->getField()->get('item_stock'),
            'taxRate' => $Item->getEntry()->getField()->get('item_tax'),
            'shippingGroupId' => $Item->getEntry()->getShippingGroup()->getId(),
            'shippingGroup' => (
                !is_null($Item->getEntry()->getShippingGroup())
                    ? $this->buildShippingGroup($Item->getEntry()->getShippingGroup())
                    : []
            ),
        ];
    }

    /**
     * 配送グループブロックの組み立て
     *
     * @param ShippingGroup $ShippingGroup
     *
     * @return array
     */
    protected function buildShippingGroup(ShippingGroup $ShippingGroup): array
    {
        return [
            'id' => $ShippingGroup->getId(),
            'status' => $ShippingGroup->getStatus(),
            'name' => $ShippingGroup->getName(),
            'fee' => $ShippingGroup->getGroupFee(),
            'calcMethod' => $ShippingGroup->getCalcMethod(),
        ];
    }
}
