<?php

namespace Acms\Plugins\ShoppingCart\GET\ShoppingForm;

use Acms\Plugins\ShoppingCart\Entities\Item;
use Acms\Plugins\ShoppingCart\Repositories\EntryRepository;
use Acms\Plugins\ShoppingCart\GET\ShoppingForm;
use App;
use Acms\Services\Facades\Session;

class Now extends ShoppingForm
{
    /**
     * 受注処理中の注文情報IDを保存する使用するセッションキー
     *
     * @var string
     */
    private const SESSION_ORDER_PROCESSING_ID_KEY = 'acms_order_processing_id_' . EID;

    /**
     * @var EntryRepository
     */
    protected EntryRepository $EntryRepository;

    /**
     * 初期化処理
     */
    protected function init()
    {
        $this->EntryRepository = App::make('shopping-cart.repository.entry');
        $this->ShoppingService = App::make('shopping-cart.service.shopping');
    }

    /**
     * 注文商品の取得
     *
     *
     * @return Item[]
     */
    protected function getItems(): array
    {

        $Entry = $this->EntryRepository->findById(EID);

        $Item = new Item();
        $Item->setEntry($Entry);

        return [$Item];
    }

    /**
     * 受注処理中の注文情報IDの取得
     *
     * @return string|null
     */
    protected function getOrderProcessingId(): ?string
    {
        $Session = Session::handle();
        return $Session->get(self::SESSION_ORDER_PROCESSING_ID_KEY) ?: null;
    }

    /**
     * 受注処理中の注文情報IDの保存
     *
     * @param string $orderProcessingId
     * @return void
     */
    protected function saveOrderProcessingId(string $orderProcessingId): void
    {
        $Session = Session::handle();
        $Session->set(self::SESSION_ORDER_PROCESSING_ID_KEY, $orderProcessingId);
        $Session->save();
    }
}
