<?php

namespace Acms\Plugins\ShoppingCart\GET\Square;

use App;
use ACMS_GET;
use ACMS_Corrector;
use Template;
use Acms\Plugins\ShoppingCart\Plugins\Square\Services\OAuth\Contracts\AbstractSquareOAuthService;

/**
 * SquareのOAuth認証の状態を表示する
 */
class OAuth extends ACMS_GET
{
    /**
     * @var AbstractSquareOAuthService
     */
    protected $SquareOAuthService;

    public function get()
    {
        $this->SquareOAuthService = App::make('square.service.oauth');
        $tpl = new Template($this->tpl, new ACMS_Corrector());

        $isAuthorized = $this->SquareOAuthService->getIsAuthorized();

        return $tpl->render(array_merge(
            [
                'isAuthorized' => $isAuthorized ? 'true' : 'false',
                'isAuthError' => $this->SquareOAuthService->getIsError() ? 'true' : 'false',
                'environment' => $this->SquareOAuthService->getEnvironment()
            ],
            $isAuthorized ? ['location' => $this->buildLocation()] : []
        ));
    }

    /**
     * locationブロックの組み立て
     *
     * @return array
     */
    protected function buildLocation(): array
    {
        // Squareからの要望でactiveな店舗のみ選択できるようにする
        $activeLocations = array_filter(
            $this->SquareOAuthService->getLocations(),
            function ($location) {
                return $location->getStatus() === 'ACTIVE';
            }
        );

        return array_map(
            function ($location) {
                return [
                    "id" => $location->getId(),
                    "name" => $location->getName()
                ];
            },
            array_values($activeLocations) // 配列のインデックスを初期化
        );
    }
}
