<?php

namespace Acms\Plugins\ShoppingCart\GET\Square;

use Acms\Plugins\ShoppingCart\Plugins\Square\Services\OAuth\Contracts\AbstractSquareOAuthService;
use Acms\Plugins\ShoppingCart\Plugins\Square\Exceptions\SquareOAuthException;
use App;
use ACMS_GET;
use Session;
use AcmsLogger;
use Common;

/**
 * OAuth認証コールバック時に実行されるモジュール
 * 認証情報を保存し、拡張アプリ設定画面にリダイレクトする
 */
class OAuthCallback extends ACMS_GET
{
    /**
     * @var AbstractSquareOAuthService
     */
    protected $SquareOAuthService;

    public function get()
    {
        $this->SquareOAuthService = App::make('square.service.oauth');

        $session = Session::handle();

        $auth = false;
        try {
            $this->SquareOAuthService->obtainToken(
                $this->Get->get('code'),
                $this->Get->get('state'),
                $session->get('square_state')
            );

            if (class_exists('AcmsLogger')) {
                AcmsLogger::info('【Square】OAuth認証に成功しました');
            }

            $auth = true;
        } catch (SquareOAuthException $e) {
            if (class_exists('AcmsLogger')) {
                AcmsLogger::error('【Square】 OAuth認証に失敗しました', Common::exceptionArray($e, [
                    'details' => $e->getDetails(),
                ]));
            } else {
                userErrorLog('ACMS Error: In ShoppingCart extension -> ' . $e->getMessage());
            }
        }

        // 標準のredirectだと、fragmentを表示できないため自作でリダイレクト
        header('Location: ' . acmsLink([
            'bid' => BID,
            'admin' => 'app_shopping_cart_index',
            'fragment' => 'acms_integration',
            'query' => ['auth' => $auth ? 'success' : 'failed']
        ]), true, 301);
        exit;
    }
}
