<?php

namespace Acms\Plugins\ShoppingCart;

use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use App;

class Hook
{
    /**
     * @var ConfigRepository
     */
    protected $Config;

    /**
     * constructor
     *
     */
    public function __construct()
    {
        $this->Config = App::make('shopping-cart.repository.config');
    }
    /**
     * 例: グローバル変数の拡張
     *
     * @param object &$globalVars
     */
    public function extendsGlobalVars(&$globalVars)
    {
        $data = [
            'PAYMENT_SERVICE' => $this->Config->get('shopping-cart_payment-service', ''),
            'SQUARE_JS' => cacheBusting(
                '/' . DIR_OFFSET . 'extension/plugins/ShoppingCart/Plugins/Square/js/dist/bundle.js',
                SCRIPT_DIR . '/extension/plugins/ShoppingCart/Plugins/Square/js/dist/bundle.js'
            ),
            'TAX_EXPRESSION_FORMAT' => $this->Config->get('shopping-cart_tax-format', ''),
            'TAX_CALC_METHOD' => $this->Config->get('shopping-cart_tax-calc-method', ''),
            'TAX_ROUNDING_METHOD' => $this->Config->get('shopping-cart_tax-rounding', ''),
            'INVOICE_NUMBER' => $this->Config->get('shopping_cart_invoice-number', ''),
            'INVOICE_NAME' => $this->Config->get('shopping_cart_invoice-name', ''),
            'SHIPPING_DATE_FROM' => $this->Config->get('shopping-cart_shipping-date-from'),
            'SHIPPING_DATE_TO' => $this->Config->get('shopping-cart_shipping-date-to'),
            'SHIPPING_FREE_SUBTOTAL' => $this->Config->get('shopping-cart_shipping-free-subtotal'),
            'SHIPPING_CALC_METHOD' => $this->Config->get('shopping-cart_calc-method', ''),
            /**
             * 拡張アプリが有効化されていない場合に、
             * 同梱テーマ内で、拡張アプリの有効化を促すアラートを表示するために使用
             * そのため、ドキュメントには載せない
             */
            'IS_SHOPPING_CART_APP_ENABLE' => 1,
        ];

        $data += [
            'SHIPPING_FREE_SUBTOTAL_FORMATTED' => (
                is_numeric($data['SHIPPING_FREE_SUBTOTAL'])
                    ? number_format(floatval($data['SHIPPING_FREE_SUBTOTAL']))
                    : ''
            )
        ];

        foreach ($data as $key => $value) {
            $globalVars->setField($key, $value);
        }
    }

    public function beforePostFire($thisModule)
    {
        $moduleName = get_class($thisModule);

        // ShoppingCart設定用
        if ($moduleName === 'ACMS_POST_Config') {
            $this->assignId(
                $thisModule,
                'shopping-cart_shipping-group-name',
                'shopping-cart_shipping-group-id',
                'group_'
            );
            $this->assignId(
                $thisModule,
                'shopping-cart_addition-name',
                'shopping-cart_addition-id',
                'addition_'
            );
        }
    }

    private function assignId(
        object $module,
        string $requiredfieldNeme,
        string $idFieldName,
        string $prefix = ''
    ) {
        $fieldAry = $module->Post->getArray($requiredfieldNeme);
        if (count($fieldAry) === 0) {
            return;
        }

        $ids = array_map(
            function (int $i) use ($module, $idFieldName, $prefix) {
                if ($module->Post->get($idFieldName, '', $i)) {
                    return $module->Post->get($idFieldName, '', $i);
                }

                return uniqid($prefix);
            },
            array_keys($fieldAry)
        );

        $module->Post->set($idFieldName, $ids);
    }
}
