<?php

namespace Acms\Plugins\ShoppingCart\POST;

use ACMS_POST_User_Update;
use ACMS_Validator_User;
use ACMS_RAM;
use SQL;
use DB;
use Acms\Services\Facades\Common;

/**
 * 閲覧ページ上からユーザー情報を編集する
 */
class MypageUpdate extends ACMS_POST_User_Update
{
    private $diff = false;

    public function post()
    {
        $this->user = $this->extract('user');
        $this->preUser = loadUser(SUID);
        $this->preField = loadUserField(SUID);

        $this->validate();
        $this->field();

        if ($this->Post->isValidAll()) {
            $this->save();
            if ((editionIsProfessional() || editionIsEnterprise())) {
                $this->old();
                if ($this->diff) {
                    $this->send();
                }
            }
        }
        return $this->Post;
    }

    protected function validate()
    {
        $validator = new ACMS_Validator_User();

        $this->user->setMethod('status', 'in', ['open', 'close']);
        $this->user->setMethod('name', 'required');
        $this->user->setMethod('mail', 'required');
        $this->user->setMethod('mail', 'email');
        $this->user->setMethod('mail', 'doubleMail', SUID);
        $this->user->setMethod('mail_magazine', 'in', ['on', 'off']);
        $this->user->setMethod('mail_mobile_magazine', 'in', ['on', 'off']);
        $this->user->setMethod('mail_mobile', 'email');
        $this->user->setMethod('url', 'url');
        $this->user->setMethod('pass', 'password');
        $this->user->setMethod('auth', 'in', ['administrator', 'editor', 'contributor', 'subscriber']);
        $this->user->setMethod('indexing', 'in', ['on', 'off']);
        $this->user->setMethod('code', 'string', isValidCode($this->user->get('code')));
        $this->user->setMethod('login_expire', 'regex', '@\d\d\d\d-\d\d-\d\d@');
        $this->user->setMethod('login_anywhere', 'in', ['on', 'off']);
        $this->user->setMethod('login_anywhere', 'anywhere', !(1
            and 'on' == $this->user->get('login_anywhere')
            and !(1
                and $validator->doubleCode($this->user->get('code'), ['uid' => SUID])
                and $validator->doubleMail($this->user->get('mail'), ['uid' => SUID])
            )
        ));
        $this->user->setMethod('user', 'operable', !!SUID);
        $this->user->validate($validator);
    }

    protected function updateUser()
    {
        $DB = DB::singleton(dsn());
        $SQL = SQL::newUpdate('user');
        $SQL->addUpdate('user_name', $this->user->get('name'));
        $SQL->addUpdate('user_code', strval($this->user->get('code')));
        $SQL->addUpdate('user_mail', $this->user->get('mail'));
        $SQL->addUpdate('user_mail_mobile', strval($this->user->get('mail_mobile')));
        $SQL->addUpdate('user_url', strval($this->user->get('url')));
        $SQL->addUpdate('user_updated_datetime', date('Y-m-d H:i:s', REQUEST_TIME));
        $SQL->addUpdate('user_locale', $this->user->get('locale'));
        $SQL->addUpdate('user_mail_magazine', $this->user->get('mail_magazine'));
        $SQL->addUpdate('user_mail_mobile_magazine', $this->user->get('mail_mobile_magazine'));

        if ($this->user->get('pass')) {
            $SQL->addUpdate('user_pass', acmsUserPasswordHash($this->user->get('pass')));
            $SQL->addUpdate('user_pass_generation', PASSWORD_ALGORITHM_GENERATION);
        }
        $this->iconQuery($SQL);

        $SQL->addWhereOpr('user_id', SUID);
        $SQL->addWhereOpr('user_blog_id', BID);
        $DB->query($SQL->get(dsn()), 'exec');

        ACMS_RAM::user(SUID, null);
    }

    protected function updateField()
    {
        Common::saveField('uid', SUID, $this->field, $this->deleteField);
    }

    protected function updateFulltext()
    {
        $DB = DB::singleton(dsn());

        $SQL = SQL::newSelect('user');
        $SQL->addWhereOpr('user_id', SUID);
        $SQL->addWhereOpr('user_blog_id', BID);
        if (!!($row = $DB->query($SQL->get(dsn()), 'row'))) {
            ACMS_RAM::user(SUID, $row);
        }
        Common::saveFulltext('uid', SUID, Common::loadUserFulltext(SUID));
        $this->Post->set('edit', 'update');
    }
}
