<?php

namespace Acms\Plugins\ShoppingCart\POST;

use Acms\Plugins\ShoppingCart\Entities\Cart;
use Acms\Plugins\ShoppingCart\Entities\Item;
use Acms\Plugins\ShoppingCart\Repositories\CartRepository;
use Acms\Plugins\ShoppingCart\Repositories\ConfigRepository;
use Acms\Plugins\ShoppingCart\Repositories\EntryRepository;
use App;
use ACMS_POST;
use Session;

/**
 * カート操作の基底クラス
 */
class ShoppingCart extends ACMS_POST
{
    /**
     * @var CartRepository
     */
    protected CartRepository $cartRepository;

    /**
     * @var ConfigRepository
     */
    protected ConfigRepository $Config;

    /**
     * @var EntryRepository
     */
    protected EntryRepository $EntryRepository;

    /**
     * @var \Acms\Services\Session\Engine
     */
    protected $Session;

    /**
     * constructor
     */
    public function __construct()
    {
        $this->cartRepository = App::make('shopping-cart.repository.cart');
        $this->Config = App::make('shopping-cart.repository.config');
        $this->EntryRepository = App::make('shopping-cart.repository.entry');
        $this->Session = Session::handle();
    }

    /**
     * カートの保存
     *
     * @param Cart $Cart
     *
     * @return void
     */
    protected function saveCart(Cart $Cart): void
    {
        $this->cartRepository->save($Cart);
    }

    /**
     * カートの取得
     *
     * @param int $bid
     *
     * @return Cart
     */
    protected function loadCart(?int $bid = BID): Cart
    {
        return $this->cartRepository->find($bid);
    }

    /**
     * ブロックをセッションに保存する
     *
     * @param string $block
     *
     * @return void
     */
    protected function setBlock(string $block): void
    {
        $this->Session->set('block', $block);
        $this->Session->save();
    }

    /**
     * カートアイテムをセッションに保存する
     *
     * @param Item $Item
     *
     * @return void
     */
    protected function setCartItem(Item $Item): void
    {
        $this->Session->set('cartItem', $Item);
        $this->Session->save();
    }
}
