<?php

namespace Acms\Plugins\ShoppingCart\POST\ShoppingCart;

use Acms\Plugins\ShoppingCart\POST\ShoppingCart;
use ACMS_Validator;
use ACMS_POST_Cache;
use AcmsLogger;

/**
 * カートから商品を削除する
 */
class RemoveItem extends ShoppingCart
{
    /**
     * @return Field_Validation|never
     */
    public function post()
    {
        $this->setBlock('remove');

        $cart = $this->loadCart();
        $cartRequest = $this->extract('cart');

        $this->validateRequest($cartRequest);

        if (!$this->Post->isValidAll()) {
            if (class_exists('AcmsLogger')) {
                AcmsLogger::notice('【Shopping Cart plugin】バリデートに失敗したため、カート内商品の削除に失敗しました', [
                    'cart' => $cartRequest->_aryV,
                ]);
            }
            return $this->Post;
        }

        $entryId = idval($cartRequest->get('eid'));
        $item = $cart->getItemByEntryId($entryId);

        if (is_null($item)) {
            $cartRequest->setValidator('eid', 'exist', false);
            if (class_exists('AcmsLogger')) {
                AcmsLogger::notice(
                    '【Shopping Cart plugin】指定したエントリーIDの商品が存在しないため、カート内商品の削除に失敗しました',
                    [
                        'entryId' => $entryId,
                    ]
                );
            }
            return $this->Post;
        }

        $cart->removeItem($item);
        $this->saveCart($cart);

        if (class_exists('AcmsLogger')) {
            AcmsLogger::info('【Shopping Cart plugin】カート内商品（' . $item->getEntry()->getTitle() . '）の削除に成功しました', [
                'entryId' => $entryId,
            ]);
        }

        $this->setCartItem($item);

        if (config('cache_clear_when_post') === 'on') {
            ACMS_POST_Cache::clearPageCache(BID);
        }

        $this->redirect(HTTP_REQUEST_URL);
    }

    /**
     * 入力値のバリデート
     *
     * @param \Field_Validation $cartRequest
     * @return void
     */
    protected function validateRequest(\Field_Validation $cartRequest)
    {
        $cartRequest->setMethod('eid', 'required');
        $cartRequest->validate(new ACMS_Validator());
    }
}
