<?php

namespace Acms\Plugins\ShoppingCart\POST\ShoppingForm;

use Acms\Plugins\ShoppingCart\Entities\Item;
use Acms\Plugins\ShoppingCart\Entities\Order;
use Acms\Plugins\ShoppingCart\Repositories\EntryRepository;
use App;
use Acms\Services\Facades\Session;
use AcmsLogger;

/**
 * 今すぐ購入フォームの確認
 */
class ConfirmNow extends Confirm
{
    /**
     * 受注処理中の注文情報IDを保存する使用するセッションキー
     *
     * @var string
     */
    private const SESSION_ORDER_PROCESSING_ID_KEY = 'acms_order_processing_id_' . EID;

    /**
     * @var Order
     */
    protected $Order;

    /**
     * @var EntryRepository
     */
    protected EntryRepository $EntryRepository;

    /**
     * constructor
     */
    public function __construct()
    {
        $this->EntryRepository = App::make('shopping-cart.repository.entry');
        parent::__construct();
    }

    /**
     * ready
     */
    protected function ready()
    {
    }

    /**
     * Run
     *
     * @return Field_Validation
     */
    public function post()
    {
        if (empty(EID)) {
            if (class_exists('AcmsLogger')) {
                AcmsLogger::notice('【Shopping Cart plugin】エントリーIDが存在しないため、購入処理を中断しました');
            } else {
                userErrorLog('Not Exists EID.');
            }
            $this->Post->set('step', 'forbidden');
            return $this->Post;
        }

        return parent::post();
    }

    /**
     * 注文商品の取得
     *
     * @return Item[]
     */
    protected function getItems(): array
    {
        $Entry = $this->EntryRepository->findById(EID);
        $quantity = intval($this->Post->getChild('field')->get('quantity', 0));

        $Item = new Item();
        $Item->setQuantity($quantity);
        $Item->setEntry($Entry);

        return [$Item];
    }

    /**
     * 受注処理中の注文情報IDの取得
     *
     * @return string|null
     */
    protected function getOrderProcessingId(): ?string
    {
        $Session = Session::handle();
        return $Session->get(self::SESSION_ORDER_PROCESSING_ID_KEY) ?: null;
    }
}
