<?php

namespace Acms\Plugins\ShoppingCart\POST\Square;

use Acms\Plugins\ShoppingCart\Plugins\Square\Exceptions\SquareOAuthException;
use Acms\Plugins\ShoppingCart\Plugins\Square\Services\OAuth\Contracts\AbstractSquareOAuthService;
use App;
use ACMS_POST;
use AcmsLogger;
use Common;

/**
 * Square OAuthの認証解除を行う
 */
class Deauth extends ACMS_POST
{
    /**
     * @var AbstractSquareOAuthService
     */
    protected $SquareOAuthService;

    /**
     * constructor
     */
    public function __construct()
    {
        $this->SquareOAuthService = App::make('square.service.oauth');
    }

    public function post()
    {
        try {
            $this->SquareOAuthService->revokeToken();

            if (class_exists('AcmsLogger')) {
                AcmsLogger::info('【Square】OAuth認証の解除に成功しました');
            }

            redirect(acmsLink([
                'bid' => BID,
                'admin' => 'app_shopping_cart_index',
            ]));
        } catch (SquareOAuthException $e) {
            if (class_exists('AcmsLogger')) {
                AcmsLogger::error('【Square】 OAuth認証の解除に失敗しました', Common::exceptionArray($e, [
                    'details' => $e->getDetails(),
                ]));
            } else {
                userErrorLog('ACMS Error: In ShoppingCart extension -> ' . $e->getMessage());
            }
        }

        return $this->Post;
    }
}
