<?php

namespace Acms\Plugins\ShoppingCart\Plugins\Square\Services\Methods;

use Acms\Plugins\ShoppingCart\Entities\Order;
use Acms\Plugins\ShoppingCart\Services\Payment\Methods\CreditCard;
use Acms\Plugins\ShoppingCart\Services\Payment\PaymentResult;
use Acms\Plugins\ShoppingCart\Services\ShoppingService;
use App;
use Field;
use Acms\Plugins\ShoppingCart\Plugins\Square\Services\SquarePaymentService;
use Acms\Plugins\ShoppingCart\Plugins\Square\Exceptions\SquarePaymentsException;

/**
 * クレジットカードの決済処理
 */
class SquareCreditCard extends CreditCard
{
    /**
     * @var Order
     */
    protected $Order;

    /**
     * @var Field
     */
    protected $Post;

    /**
     * @var SquarePaymentService
     */
    protected $SquarePaymentService;

    /**
     * @var ShoppingService
     */
    protected $ShoppingService;

    /**
     * constructor
     */
    public function __construct()
    {
        $this->SquarePaymentService = App::make('square.service.payment');
        $this->ShoppingService = App::make('shopping-cart.service.shopping');
    }

    /**
     * {@inheritdoc}
     */
    public function getCode(): string
    {
        return 'square-credit-card';
    }

    /**
     * クレジットカードの有効性チェックを行う
     *
     * @return PaymentResult
     */
    public function verify(): PaymentResult
    {
        $PaymentResult = new PaymentResult();
        $PaymentResult->setSuccess(true);

        return $PaymentResult;
    }

    /**
     * クレジットカードの決済処理を行う
     *
     * @return PaymentResult
     */
    public function checkout(): PaymentResult
    {
        $PaymentResult = new PaymentResult();

        try {
            $Order = $this->SquarePaymentService->createOrder($this->Order);
            $field = $this->Post->getChild('field');

            $nonce = $field->get('square-nonce');
            $note = $field->isExists('square-note') ? $field->get('square-note') : 'a-blog cms による決済';
            $Payment = $this->SquarePaymentService->payment(
                $this->Order->getTotal(),
                $Order->getId(),
                $nonce,
                $this->Order->getProcessingId(),
                $note
            );
            $PaymentResult->setSuccess(true);

            // メール用変数セット
            $this->setMailVars($field, $Payment);
        } catch (SquarePaymentsException $e) {
            $PaymentResult->setSuccess(false);
            $PaymentResult->setErrors(['auth']);
            $PaymentResult->setMessage($e->getMessage());
            $PaymentResult->setDetails($e->getDetails());
        }

        return $PaymentResult;
    }

    /**
     * メール用の変数を追加する
     *
     * @param Field $field
     * @param \Square\Models\Payment $Payment
     */
    private function setMailVars(Field $field, \Square\Models\Payment $Payment): void
    {
        $field->set('square-last4', $Payment->getCardDetails()->getCard()->getLast4());
        $field->set('square-receiptNumber', $Payment->getReceiptNumber());
        $field->set('square-receiptUrl', $Payment->getReceiptUrl());
    }
}
