<?php

namespace Acms\Plugins\ShoppingCart\Plugins\Square\Util;

use App;

/**
 * Squareのユーティリティクラス
 */
class SquareUtil
{
    /**
     * Squareからのリクエストかどうかチェックする
     * https://developer.squareup.com/docs/webhooks/step3validate
     *
     * @param string $signature x-square-hmacsha256-signatureヘッダーの値
     * @param string $body
     *
     * @return bool
     */
    public static function isFromSquare(string $signature, string $body): bool
    {
        $Config = App::make('shopping-cart.repository.config');

        $hash = hash_hmac(
            'sha256',
            HTTP_REQUEST_URL . $body,
            $Config->get('square_webhook_signature_key', ''),
            true
        );
        return base64_encode($hash) === $signature;
    }

    /**
     * Squareのエラーレスポンスをパースする
     *
     * @param string $methodName エラーが発生したメソッド名
     * @param \Square\Models\Error[] $errors
     *
     * @return string エラーメッセージ
     */
    public static function parseResponseError(string $methodName, array $errors): string
    {
        $code = $errors[0]->getCode();
        $category = $errors[0]->getCategory();
        $detail = $errors[0]->getDetail();

        return 'Error Processing Request: ' . $methodName . ' failed!, ' . $code . ', ' . $category . ', ' . $detail;
    }
}
