<?php

namespace Acms\Plugins\ShoppingCart\Plugins\Square\Webhook;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Acms\Plugins\ShoppingCart\Plugins\Square\Util\SquareUtil;

class Controller
{
    /**
     * @var Action
     */
    protected $Action;

    /**
     * WebhookのEventとActionのマッピング
     *
     * @var array
     */
    protected $webhooks = [
        'oauth.authorization.revoked' => 'revokeOAuthToken'
    ];

    public function __construct()
    {
        $this->Action = new Action();
    }

    /**
     * WebhookのEventによって処理を分ける
     * @see https://developer.squareup.com/reference/square/webhooks
     *
     * @param Request $request
     * @param Response $response
     *
     * @return Response
     */
    public function handle(Request $request, Response $response)
    {
        $signature = $request->getHeader('X-Square-HmacSha256-Signature')[0];
        $body = $request->getParsedBody();

        if (!SquareUtil::isFromSquare($signature, json_encode($body))) {
            userErrorLog('ACMS Error: Illegal Event Notification.');
            return $response->withStatus(403);
        }

        $method = $this->webhooks[$body['type']];

        if (!method_exists($this->Action, $method)) {
            userErrorLog('ACMS Error: Not Implemented Webhook method.');
            return $response->withStatus(404);
        }

        $this->Action->{$method}($request, $response);

        return $response->withStatus(204);
    }
}
