<?php

namespace Acms\Plugins\ShoppingCart\Services;

class Encrypter
{
    /**
     * 暗号化に使用するメソッド
     */
    public $cipher;

    /**
     * constractor
     */
    public function __construct(?string $cipher = 'aes-128-gcm')
    {
        $this->cipher = $cipher;
    }

    /**
     * AESによる暗号化
     *
     * @param string $data
     * @param string $passphrase
     *
     * @return string|false
     */
    public function encrypt($data, $passphrase)
    {
        if (!in_array($this->cipher, openssl_get_cipher_methods())) {
            return false;
        }

        $ivlen = openssl_cipher_iv_length($this->cipher);
        $iv = openssl_random_pseudo_bytes($ivlen);

        $encrypted = openssl_encrypt($data, $this->cipher, $passphrase, OPENSSL_RAW_DATA, $iv, $tag);

        return base64_encode($iv . $tag . $encrypted);
    }

    /**
     * AESで暗号化されたデータを複合する
     *
     * @param string $data
     * @param string $passphrase
     *
     * @return string|false
     */
    public function decrypt($data, $passphrase)
    {
        if (!in_array($this->cipher, openssl_get_cipher_methods())) {
            return false;
        }

        $decoded = base64_decode($data);

        $ivlen = openssl_cipher_iv_length($this->cipher);
        $iv = substr($decoded, 0, $ivlen);
        $tag = substr($decoded, $ivlen, 16);
        $encrypted = substr($decoded, $ivlen + 16);

        return openssl_decrypt($encrypted, $this->cipher, $passphrase, OPENSSL_RAW_DATA, $iv, $tag);
    }
}
