<?php

namespace Acms\Plugins\ShoppingCart\Services\Payment\Methods;

use App;
use Acms\Plugins\ShoppingCart\Entities\Order;
use Acms\Plugins\ShoppingCart\Services\Payment\PaymentResult;
use Acms\Plugins\ShoppingCart\Services\Payment\PaymentMethodInterface;

class CreditCardAdapter extends CreditCard
{
    private CreditCard $CreditCard;

    public function __construct()
    {
        $Config = App::make('shopping-cart.repository.config');
        $this->CreditCard = CreditCardFactory::singleton()->create($Config->get('shopping-cart_payment-service', ''));
    }

    /**
     * {@inheritdoc}
     */
    public function getCode(): string
    {
        return $this->CreditCard->getCode();
    }

    /**
     * {@inheritdoc}
     */
    public function verify(): PaymentResult
    {
        return $this->CreditCard->verify();
    }

    /**
     * {@inheritdoc}
     */
    public function checkout(): PaymentResult
    {
        return $this->CreditCard->checkout();
    }

    /**
     * {@inheritdoc}
     */
    public function setOrder(Order $Order): PaymentMethodInterface
    {
        return $this->CreditCard->setOrder($Order);
    }


    /**
     * {@inheritdoc}
     */
    public function setPost(\Field $Post): PaymentMethodInterface
    {
        return $this->CreditCard->setPost($Post);
    }

    public function __call($name, $args)
    {
        if (method_exists($this->CreditCard, $name)) {
            return call_user_func_array([$this->CreditCard, $name], $args);
        }
    }
}
