<?php

namespace Acms\Plugins\ShoppingCart\Services\Payment;

use Acms\Plugins\ShoppingCart\Entities\Order;
use Field;

/**
 * PaymentMethodInterface
 *
 * 必要に応じて決済手段ごとに実装する
 */
interface PaymentMethodInterface
{
    /**
     * 決済手段のコードを返します.
     *
     * @return string
     */
    public function getCode(): string;

    /**
     * 決済の妥当性を検証し, 検証結果を返します。
     *
     * 主にクレジットカードの有効性チェック等を実装します。
     *
     * @return PaymentResult
     */
    public function verify(): PaymentResult;

    /**
     * 決済の実行
     *
     * 主に決済の確定処理を実装します。
     *
     * @return PaymentResult
     */
    public function checkout(): PaymentResult;

    /**
     * この決済を使用する Order を設定します.
     *
     * @param Order $Order
     *
     * @return PaymentMethodInterface
     */
    public function setOrder(Order $Order): PaymentMethodInterface;

    /**
     *
     *
     * @param Field $Post
     *
     * @return PaymentMethodInterface
     */
    public function setPost(Field $Post): PaymentMethodInterface;
}
