<?php

namespace Acms\Plugins\ShoppingCart\Services\Payment;

/**
 * 決済結果のクラス
 */
class PaymentResult
{
    /**
     * @var array
     */
    private $errors = [];

    /**
     * エラーメッセージ
     *
     * @var string
     */
    private $message = '';

    /**
     * @var boolean
     */
    private $success;

    /**
     * @var array
     */
    private $details = [];

    /**
     * 決済が成功したかどうかを設定します
     *
     * 決済が成功した場合は true, 失敗した場合は false を設定します
     *
     * @param boolean $success
     *
     * @return PaymentResult
     */
    public function setSuccess($success)
    {
        $this->success = $success;

        return $this;
    }

    /**
     * 決済が成功したかどうか
     *
     * 決済が成功した場合 true
     *
     * @return boolean
     */
    public function isSuccess()
    {
        return $this->success;
    }

    /**
     * 決済が失敗した場合のエラーの配列を返します
     *
     * @return array
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * 決済が失敗した場合のエラーの配列を設定します
     *
     * @param array $errors
     *
     * @return PaymentResult
     */
    public function setErrors(array $errors)
    {
        $this->errors = $errors;

        return $this;
    }

    /**
     * 決済が失敗した場合のエラーメッセージを返します
     *
     * @return string
     */
    public function getMessage(): string
    {
        return $this->message;
    }

    /**
     * 決済が失敗した場合のエラーの配列を設定します
     *
     * @param string $message
     *
     * @return PaymentResult
     */
    public function setMessage(string $message)
    {
        $this->message = $message;

        return $this;
    }

    /**
     * 決済が失敗した場合の詳細情報を返します
     *
     * @return array
     */
    public function getDetails()
    {
        return $this->details;
    }

    /**
     * 決済が失敗した場合の詳細情報を設定します
     *
     * @param array $details
     */
    public function setDetails(array $details)
    {
        $this->details = $details;
    }
}
