import domContentLoaded from 'dom-content-loaded';
import $ from 'jquery';
import 'slick-carousel';

import './polyfill';
import topPage from './top';

/**
 * スタイルの読み込み
 */
import 'normalize.css/normalize.css';
import 'slick-carousel/slick/slick.scss';
import '../scss/site.scss';

/* eslint func-names: 0 */

/**
 * shopping cart
 */
import(/* webpackChunkName: "shopping-cart" */ './shopping-cart').then(({ default: app }) => {
  domContentLoaded(() => {
    app(document);
  });
});

topPage(); // トップページ

/**
 * Content Ready
 */
domContentLoaded(() => {
  /**
   * common
   */
  $.fn.delayAddClass = function (className, delay) {
    return $(this)
      .delay(delay)
      .queue(function (next) {
        $(this).addClass(className);
        next();
      });
  };
  $.fn.delayRemoveClass = function (className, delay) {
    return $(this)
      .delay(delay)
      .queue(function (next) {
        $(this).removeClass(className);
        next();
      });
  };

  // オフキャンバスメニュー
  const $mobileNavTrigger = $('.js-mobile-nav-btn');
  const $mobileNavContents = $('.js-mobile-nav');
  $($mobileNavTrigger).on('click', () => {
    $('body').toggleClass('is-locked');
    const $mobileNavAttr = $($mobileNavTrigger).attr('aria-expanded');
    if ($mobileNavAttr === 'true') {
      $($mobileNavTrigger).attr('aria-expanded', false);
      $($mobileNavTrigger).attr('aria-label', 'メニューを開く');
      $($mobileNavContents).delayRemoveClass('is-opened', 1).delayRemoveClass('is-active', 100);
    } else {
      $($mobileNavTrigger).attr('aria-expanded', true);
      $($mobileNavTrigger).attr('aria-label', 'メニューを閉じる');
      $($mobileNavContents).delayAddClass('is-active', 1).delayAddClass('is-opened', 100);
    }
  });

  // 郵便番号の「-」の挿入
  $('.js-insert-hyphen').on('blur', () => {
    function insertStr(str, index, insert) {
      return str.slice(0, index) + insert + str.slice(index, str.length);
    }
    const zip = $(this).val();
    if (typeof zip !== 'string') {
      return;
    }
    if (zip.length > 6 && !zip.match('-')) {
      const zipHyphen = insertStr(zip, 3, '-');
      $(this).val(zipHyphen);
    }
  });

  // カルーセル
  if ($('.js-slider').length > 0) {
    const sliderMark = '.js-slider'; // スライダー
    const thumbnailItemMark = '.js-slider-thumbs-item';

    $(thumbnailItemMark).each((index, el) => {
      $(el).attr('data-index', index);
    });

    $(sliderMark).on('init', () => {
      const itemIndex = $('.slick-slide.slick-current').attr('data-slick-index');
      $(`${thumbnailItemMark}[data-index="${itemIndex}"]`).addClass('js-slider-thumbs-current');
    });

    $(sliderMark).slick({
      autoplay: false,
      arrows: false,
      fade: true,
      infinite: false,
      adaptiveHeight: true,
    });

    $(thumbnailItemMark).on('click', (event) => {
      const $this = $(event.currentTarget);
      const index = $this.attr('data-index');
      $(sliderMark).slick('slickGoTo', index, false);
    });

    $(sliderMark).on('beforeChange', (event, slick, currentSlide, nextSlide) => {
      $(thumbnailItemMark).each((index, element) => {
        const $this = $(element);
        $this.removeClass('is-current');
      });
      $(`${thumbnailItemMark}[data-index="${nextSlide}"]`).addClass('is-current');
    });
  }

  // 2重送信対策
  ((selector = '.js-disable-on-submit') => {
    let submitter = null;
    let input = null;
    const forms = document.querySelectorAll(selector);

    const handleSubmit = (event) => {
      submitter = event.submitter;
      const form = event.target;

      if (!submitter) {
        return;
      }

      submitter.disabled = true;

      if (submitter.name) {
        input = document.createElement('input');
        input.type = 'hidden';
        input.name = submitter.name;
        form.appendChild(input);
      }
    };

    const handleDocumentSubmit = (event) => {
      // 送信キャンセルされた場合は、disabledを解除
      if ([].includes.call(forms, event.target) && event.defaultPrevented) {
        submitter.disabled = false;
        if (input) {
          input.remove();
          input = null;
        }
      }
    };

    forms.forEach((form) => {
      form.addEventListener('submit', handleSubmit);
    });
    document.addEventListener('submit', handleDocumentSubmit);
  })();
});
