import { ref, computed } from 'vue';

export default {
  name: 'QuantitySelect',
  template: `
    <select v-if="!isMore" :name="name" :value="quantity" @change="handleChange($event.target.value)">
      <option v-for="number in max" :value="number" :key="number">{{ number }}</option>
      <option :value="max + 1" v-if="maxQuantity > max">{{ max + 1 }}+</option>
    </select>
    <input v-else type="number" :name="name" :value="quantity" min="1" :max="maxQuantity" @input="$emit('update:quantity', parseInt($event.target.value, 10))" />
  `,
  props: {
    name: {
      default: 'quantity',
      type: String,
    },
    quantity: {
      default: 1,
      type: Number,
    },
    maxQuantity: {
      default: 10,
      type: Number,
    },
    limit: {
      default: 10,
      type: Number,
    },
  },
  emits: ['update:quantity'],
  setup(props, { emit }) {
    const isMore = ref(false);
    // セレクトボックスで入力できる値の最大値
    const max = computed(() => (Math.min(props.maxQuantity, props.limit)));

    const handleChange = (number) => {
      // to input from select
      if (number > max.value) {
        isMore.value = true;
      }

      emit('update:quantity', parseInt(number, 10));
    };

    return {
      isMore,
      max,
      handleChange,
    };
  },
};
