import { ref, computed } from 'vue';
import { useStore } from 'vuex';

import ShoppingCartItem from './ShoppingCartItem';
import { formatMoney } from '../util';

export default {
  name: 'ShoppingCartList',
  template: '#ShoppingCartList',
  components: {
    ShoppingCartItem,
  },
  setup() {
    const editing = ref(null);
    const setEditing = (eid) => {
      editing.value = eid || null;
    };

    const store = useStore();

    const isLoading = computed(() => store.state.cart.isLoading);
    const isValidating = computed(() => store.state.cart.isValidating);
    const items = computed(() => store.getters['cart/items']);
    const taxes = computed(() => store.getters['cart/taxes']);
    const subtotal = computed(() => store.getters['cart/subtotal']);

    const validate = async () => {
      store.commit('cart/SET_VALIDATING', true);
      await store.dispatch('cart/getCart');
      store.commit('cart/SET_VALIDATING', false);
    };

    const handleAfterUpdate = () => {
      setEditing(null);
      validate();
    };

    const handleAfterRemove = () => {
      if (items.value.length === 0) {
        return window.location.reload();
      }

      return validate();
    };

    return {
      editing,
      isLoading,
      isValidating,
      items,
      taxes,
      subtotal,
      formatMoney,
      setEditing,
      handleAfterUpdate,
      handleAfterRemove,
    };
  },
};
