import accounting from 'accounting';

/**
 * 日本円にフォーマットする
 *
 * @param {number} number
 *
 * @returns {string}
 */
const formatMoney = (number) => accounting.formatMoney(number, '¥', 0);

/**
 * 比率と高さから幅を取得する
 *
 * @param {number} ratio {xxxx@ratio}
 * @param {number} height 高さ
 *
 * @returns {number} 幅
 */
const getWidthFromRatio = (ratio, height) => {
  if (!ratio || !height) return '';

  return Math.round(height * ratio);
};

/**
 * 比率と幅から高さを取得する
 *
 * @param {number} ratio {xxxx@ratio}
 * @param {number} width 幅
 *
 * @returns {number} 高さ
 */
const getHeightFromRatio = (ratio, width) => {
  if (!ratio || !width) return '';

  return Math.round(width / ratio);
};

/**
 * 値がオブジェクトかどうかを判定する
 *
 * @param {*} value
 *
 * @returns {boolean} 高さ
 */
const isObject = (value) => value !== null && typeof value === 'object';

export {
  formatMoney,
  getWidthFromRatio,
  getHeightFromRatio,
  isObject,
};
